import logging
import os

import requests

from pullbug.logger import PullBugLogger
from pullbug.messages import Messages

GITLAB_API_KEY = os.getenv('GITLAB_API_KEY')
GITLAB_API_URL = os.getenv('GITLAB_API_URL', 'https://gitlab.com/api/v4')
IGNORE_WIP = os.getenv('IGNORE_WIP')
GITLAB_HEADERS = {
    'authorization': f'Bearer {GITLAB_API_KEY}'
}
LOGGER = logging.getLogger(__name__)


class GitlabBug():
    @classmethod
    def run(cls, gitlab_scope='all', gitlab_state='opened', wip=False, discord=False, slack=False, rocketchat=False):
        """Run the logic to get MR's from GitLab and
        send that data via message.
        """
        PullBugLogger._setup_logging(LOGGER)
        merge_requests = cls.get_merge_requests(gitlab_scope, gitlab_state)

        if merge_requests == []:
            message = 'No merge requests are available from GitLab.'
            LOGGER.info(message)
            return message

        message_preamble = '\n:bug: *The following merge requests on GitLab are still open and need your help!*\n'
        messages, discord_messages = cls.iterate_merge_requests(merge_requests, wip, discord, slack, rocketchat)
        messages.insert(0, message_preamble)
        discord_messages.insert(0, message_preamble)
        if discord:
            Messages.send_discord_message(discord_messages)
        if slack:
            Messages.send_slack_message(messages)
        if rocketchat:
            Messages.send_rocketchat_message(messages)
        LOGGER.info(messages)

    @classmethod
    def get_merge_requests(cls, gitlab_scope, gitlab_state):
        """Get all repos of the GITLAB_API_URL.
        """
        LOGGER.info('Bugging GitLab for merge requests...')
        try:
            response = requests.get(
                f"{GITLAB_API_URL}/merge_requests?scope={gitlab_scope}&state={gitlab_state}&per_page=100",
                headers=GITLAB_HEADERS
            )
            LOGGER.debug(response.text)
            LOGGER.info('GitLab merge requests retrieved!')
            if 'does not have a valid value' in response.text:
                error = f'Could not retrieve GitLab merge requests due to bad parameter: {gitlab_scope} | {gitlab_state}.'  # noqa
                LOGGER.error(error)
                raise ValueError(error)
        except requests.exceptions.RequestException as response_error:
            LOGGER.error(
                f'Could not retrieve GitLab merge requests: {response_error}'
            )
            raise requests.exceptions.RequestException(response_error)
        return response.json()

    @classmethod
    def iterate_merge_requests(cls, merge_requests, wip, discord, slack, rocketchat):
        """Iterate through each merge request of a repo
        and build the message array.
        """
        message_array = []
        discord_message_array = []
        for merge_request in merge_requests:
            # TODO: There is a "work_in_progress" key in the response
            # that could be used? https://docs.gitlab.com/ee/api/merge_requests.html
            if not wip and 'WIP' in merge_request['title'].upper():
                continue
            else:
                message, discord_message = Messages.prepare_gitlab_message(merge_request, discord, slack, rocketchat)
                message_array.append(message)
                discord_message_array.append(discord_message)
        return message_array, discord_message_array



import logging
import math
import os
import re

import requests
import slack

DISCORD_WEBHOOK_URL = os.getenv('DISCORD_WEBHOOK_URL')
ROCKET_CHAT_URL = os.getenv('ROCKET_CHAT_URL')
SLACK_BOT_TOKEN = os.getenv('SLACK_BOT_TOKEN')
SLACK_CHANNEL = os.getenv('SLACK_CHANNEL')
GITLAB_API_URL = os.getenv('GITLAB_API_URL', 'https://gitlab.com/api/v4')
LOGGER = logging.getLogger(__name__)


class Messages():
    @classmethod
    def send_discord_message(cls, message):
        """Send a Discord message.

        Discord has a hard limit of 2000 characters per message,
        as such, we break up the messages into batches, allow for
        breathing room, and send each batch of messages separately.
        """
        num_of_messages = len(message)
        max_per_batch = 6
        i = 1
        new_cutoff = max_per_batch
        old_cutoff = 0
        while i <= math.ceil(num_of_messages / max_per_batch):
            i += 1
            batch_message = ''.join(message[old_cutoff:new_cutoff])
            new_cutoff += max_per_batch
            old_cutoff += max_per_batch
            try:
                requests.post(DISCORD_WEBHOOK_URL, json={'content': batch_message})
                LOGGER.info('Discord message sent!')
            except requests.exceptions.RequestException as discord_error:
                LOGGER.error(f'Could not send Discord message: {discord_error}')
                raise requests.exceptions.RequestException(discord_error)

    @classmethod
    def send_rocketchat_message(cls, message):
        """Send a Rocket Chat message

        We truncate the message at 40,000 characters to match Slack
        and improve performance. RC doesn't specify a char limit.
        """
        rocketchat_message = ''.join(message)[:40000]
        try:
            requests.post(ROCKET_CHAT_URL, json={'text': rocketchat_message})
            LOGGER.info('Rocket Chat message sent!')
        except requests.exceptions.RequestException as rc_error:
            LOGGER.error(f'Could not send Rocket Chat message: {rc_error}')
            raise requests.exceptions.RequestException(rc_error)

    @classmethod
    def send_slack_message(cls, message):
        """Send Slack messages via a bot.

        Slack truncates messages after 40,000 characters so
        we truncate there before sending the request.
        """
        slack_message = ''.join(message)[:40000]
        slack_client = slack.WebClient(SLACK_BOT_TOKEN)
        try:
            slack_client.chat_postMessage(
                channel=SLACK_CHANNEL,
                text=slack_message
            )
            LOGGER.info('Slack message sent!')
        except slack.errors.SlackApiError as slack_error:
            LOGGER.error(f'Could not send Slack message: {slack_error}')
            raise slack.errors.SlackApiError(
                slack_error.response["ok"], slack_error.response['error']
            )

    @classmethod
    def prepare_github_message(cls, pull_request, discord, slack, rocketchat):
        """Prepares a GitHub message with a single pull request's data.
        This will then be appended to an array of messages.

        Slack & RocketChat can use the same format while Discord requires
        some tweaking.
        """
        # TODO: Check requested_reviewers array also
        description_max_length = 120
        users = ''
        discord_users = ''
        if pull_request.get('assignees'):
            for assignee in pull_request['assignees']:
                discord_user = f"{assignee['login']} (<{assignee['html_url']}>)"
                user = f"<{assignee['html_url']}|{assignee['login']}>"
                users += user + ' '
                discord_users += discord_user + ' '
        else:
            users = 'NA'

        description = (pull_request.get('body')[:description_max_length] +
                       '...') if len(pull_request['body']) > description_max_length else pull_request.get('body')
        message = (
            f"\n:arrow_heading_up: *Pull Request:* <{pull_request['html_url']}|{pull_request['title']}>"
            f"\n*Repo:* <{pull_request['base']['repo']['html_url']}|{pull_request['base']['repo']['name']}>"
            f"\n*Description:* {description}\n*Waiting on:* {users}\n"
        )
        discord_message = (
            f"\n:arrow_heading_up: **Pull Request:** {pull_request['title']} (<{pull_request['html_url']}>)"
            f"\n**Repo:** {pull_request['base']['repo']['name']} (<{pull_request['base']['repo']['html_url']}>)"
            f"\n**Description:** {description}\n**Waiting on:** {discord_users}\n"
        )

        return message, discord_message

    @classmethod
    def prepare_gitlab_message(cls, merge_request, discord, slack, rocketchat):
        """Prepare a GitLab message with a single merge request's data.
        This will then be appended to an array of messages.

        Slack & RocketChat can use the same format while Discord requires
        some tweaking.
        """
        description_max_length = 120
        users = ''
        discord_users = ''
        re_repo_name = re.compile('(?P<group_name>[\\w-]+)/(?P<repo_name>[\\w-]+)!(?P<merge_request_num>\\d+)')
        re_match = re_repo_name.search(merge_request['references']['full'])
        if merge_request.get('assignees'):
            for assignee in merge_request['assignees']:
                user = f"<{assignee['web_url']}|{assignee['username']}>"
                discord_user = f"{assignee['username']} (<{assignee['web_url']}>)"
                users += user + ' '
                discord_users += discord_user + ' '
        else:
            users = 'NA'

        description = (merge_request.get('description')[:description_max_length] +
                       '...') if len(merge_request['description']) > description_max_length else merge_request.get('description')  # noqa
        message = (
            f"\n:arrow_heading_up: *Merge Request:* <{merge_request['web_url']}|{merge_request['title']}>"
            f"\n*Repo:* <{GITLAB_API_URL}/{re_match['group_name']}/{re_match['repo_name']}|{re_match['repo_name']}>"
            f"\n*Description:* {description}\n*Waiting on:* {users}\n"
        )
        discord_message = (
            f"\n:arrow_heading_up: **Pull Request:** {merge_request['title']} (<{merge_request['web_url']}>)"
            f"\n**Repo:** {re_match['repo_name']} (<{GITLAB_API_URL}/{re_match['group_name']}/{re_match['repo_name']}>)"
            f"\n**Description:** {description}\n**Waiting on:** {discord_users}\n"
        )

        return message, discord_message



from test.conftest import MockResponse

import mock
import pytest
import requests
from pullbug.gitlab_bug import GitlabBug

GITLAB_API_KEY = '123'
MOCK_URL = 'http://mock-url.com'


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.messages.Messages.send_rocketchat_message')
@mock.patch('pullbug.messages.Messages.send_slack_message')
@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests', return_value=[['mock-message'], ['mock-message']])
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_no_slack_or_rocketchat_messages(mock_request, mock_logger, mock_headers,
                                             mock_get_merge_requests, mock_iterate_merge_requests,
                                             mock_prepare_message, mock_slack, mock_rocketchat,
                                             _mock_gitlab_scope, _mock_gitlab_state):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, False, False)

    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_slack.assert_not_called()
    mock_rocketchat.assert_not_called()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.messages.Messages.send_discord_message')
@mock.patch('pullbug.messages.Messages.send_slack_message')
@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests', return_value=[['mock-message'], ['mock-message']])
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_discord_message(mock_request, mock_logger, mock_headers,
                             mock_get_merge_requests, mock_iterate_merge_requests,
                             mock_prepare_message, mock_slack, mock_discord,
                             _mock_gitlab_scope, _mock_gitlab_state):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, True, False, False)

    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_discord.assert_called_once()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.messages.Messages.send_rocketchat_message')
@mock.patch('pullbug.messages.Messages.send_slack_message')
@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests', return_value=[['mock-message'], ['mock-message']])
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_slack_message(mock_request, mock_logger, mock_headers,
                           mock_get_merge_requests, mock_iterate_merge_requests,
                           mock_prepare_message, mock_slack, mock_rocketchat,
                           _mock_gitlab_scope, _mock_gitlab_state):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, True, False)

    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_slack.assert_called_once()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.messages.Messages.send_rocketchat_message')
@mock.patch('pullbug.messages.Messages.send_slack_message')
@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests', return_value=[['mock-message'], ['mock-message']])
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests')
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_rocketchat_message(mock_request, mock_logger, mock_headers,
                                mock_get_merge_requests, mock_iterate_merge_requests,
                                mock_prepare_message, mock_slack, mock_rocketchat,
                                _mock_gitlab_scope, _mock_gitlab_state):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, False, True)

    mock_logger.info.assert_called()
    mock_get_merge_requests.assert_called_once()
    mock_iterate_merge_requests.assert_called()
    mock_slack.assert_not_called()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.messages.Messages.send_rocketchat_message')
@mock.patch('pullbug.messages.Messages.send_slack_message')
@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message')
@mock.patch('pullbug.gitlab_bug.GitlabBug.iterate_merge_requests', return_value=[['mock-message'], ['mock-message']])
@mock.patch('pullbug.gitlab_bug.GitlabBug.get_merge_requests', return_value=[])
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_run_no_returned_merge_requests(mock_request, mock_logger, mock_headers,
                                        mock_get_merge_requests, mock_iterate_merge_requests,
                                        mock_prepare_message, mock_slack, mock_rocketchat,
                                        _mock_gitlab_scope, _mock_gitlab_state):
    GitlabBug.run(_mock_gitlab_scope, _mock_gitlab_state, False, False, False, False)

    mock_logger.info.assert_called_with('No merge requests are available from GitLab.')
    mock_iterate_merge_requests.assert_not_called()
    mock_slack.assert_not_called()
    mock_rocketchat.assert_not_called()


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get')
def test_get_merge_requests_success(mock_request, mock_logger, mock_headers, _mock_gitlab_scope,
                                    _mock_gitlab_state, _mock_url):
    # TODO: Mock this request better and assert additional values
    GitlabBug.get_merge_requests(_mock_gitlab_scope, _mock_gitlab_state)

    mock_request.assert_called_once_with(
        f'{_mock_url}/merge_requests?scope={_mock_gitlab_scope}&state={_mock_gitlab_state}&per_page=100',
        headers=mock_headers
    )
    assert mock_logger.info.call_count == 2


@mock.patch('pullbug.gitlab_bug.GITLAB_API_KEY', GITLAB_API_KEY)
@mock.patch('pullbug.gitlab_bug.GITLAB_API_URL', MOCK_URL)
@mock.patch('pullbug.gitlab_bug.GITLAB_HEADERS')
@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get', return_value=MockResponse(text='does not have a valid value'))
def test_get_merge_value_exception(mock_request, mock_logger, mock_headers, _mock_gitlab_scope,
                                   _mock_gitlab_state, _mock_url):
    with pytest.raises(ValueError):
        GitlabBug.get_merge_requests(_mock_gitlab_scope, _mock_gitlab_state)

    mock_logger.error.assert_called_once_with(
        f'Could not retrieve GitLab merge requests due to bad parameter: {_mock_gitlab_scope} | {_mock_gitlab_state}.'
    )


@mock.patch('pullbug.gitlab_bug.LOGGER')
@mock.patch('requests.get', side_effect=requests.exceptions.RequestException('mock-error'))
def test_get_repos_exception(mock_request, mock_logger, _mock_gitlab_scope, _mock_gitlab_state):
    with pytest.raises(requests.exceptions.RequestException):
        GitlabBug.get_merge_requests(_mock_gitlab_scope, _mock_gitlab_state)

    mock_logger.error.assert_called_once_with(
        'Could not retrieve GitLab merge requests: mock-error'
    )


@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message', return_value=[['mock-message'], ['mock-message']])
def test_iterate_merge_requests_wip_title(mock_prepare_message, _mock_merge_request):
    _mock_merge_request['title'] = 'wip: mock-merge-request'
    mock_merge_requests = [_mock_merge_request]
    GitlabBug.iterate_merge_requests(mock_merge_requests, True, False, False, False)

    mock_prepare_message.assert_called_once()


@mock.patch('pullbug.gitlab_bug.Messages.prepare_gitlab_message')
def test_iterate_merge_requests_wip_setting_absent(mock_prepare_message, _mock_merge_request):
    _mock_merge_request['title'] = 'wip: mock-merge-request'
    mock_merge_requests = [_mock_merge_request]
    GitlabBug.iterate_merge_requests(mock_merge_requests, False, False, False, False)

    mock_prepare_message.assert_not_called()



import mock
import pytest
import requests
import slack
from pullbug.messages import Messages


@mock.patch('pullbug.messages.DISCORD_WEBHOOK_URL', 'https://discord.com/api/webhooks/channel_id/webhook_id')
@mock.patch('pullbug.messages.LOGGER')
@mock.patch('requests.post')
def test_discord_success(mock_request, mock_logger):
    message = 'mock message'
    Messages.send_discord_message([message])

    mock_request.assert_called_once_with(
        'https://discord.com/api/webhooks/channel_id/webhook_id', json={'content': message}
    )
    mock_logger.info.assert_called_once_with('Discord message sent!')


@mock.patch('pullbug.messages.LOGGER')
@mock.patch('requests.post', side_effect=requests.exceptions.RequestException('mock-error'))
def test_discord_exception(mock_request, mock_logger):
    message = 'mock message'
    with pytest.raises(requests.exceptions.RequestException):
        Messages.send_discord_message(message)

    mock_logger.error.assert_called_once_with(
        'Could not send Discord message: mock-error'
    )


@mock.patch('pullbug.messages.ROCKET_CHAT_URL', 'http://mock-url.com')
@mock.patch('pullbug.messages.LOGGER')
@mock.patch('requests.post')
def test_rocket_chat_success(mock_request, mock_logger):
    message = 'mock message'
    Messages.send_rocketchat_message(message)

    mock_request.assert_called_once_with(
        'http://mock-url.com', json={'text': message}
    )
    mock_logger.info.assert_called_once_with('Rocket Chat message sent!')


@mock.patch('pullbug.messages.LOGGER')
@mock.patch('requests.post', side_effect=requests.exceptions.RequestException('mock-error'))
def test_rocket_chat_exception(mock_request, mock_logger):
    message = 'mock message'
    with pytest.raises(requests.exceptions.RequestException):
        Messages.send_rocketchat_message(message)

    mock_logger.error.assert_called_once_with(
        'Could not send Rocket Chat message: mock-error'
    )


@mock.patch('pullbug.messages.SLACK_CHANNEL', 'mock-channel')
@mock.patch('pullbug.messages.SLACK_BOT_TOKEN', '123')
@mock.patch('pullbug.messages.LOGGER')
@mock.patch('slack.WebClient.chat_postMessage')
def test_slack_success(mock_slack, mock_logger):
    message = 'mock message'
    Messages.send_slack_message(message)

    mock_slack.assert_called_once_with(channel='mock-channel', text=message)
    mock_logger.info.assert_called_once_with('Slack message sent!')


@mock.patch('pullbug.messages.LOGGER')
@mock.patch('slack.WebClient.chat_postMessage',
            side_effect=slack.errors.SlackApiError(
                message='The request to the Slack API failed.',
                response={'ok': False, 'error': 'not_authed'}
            ))
def test_slack_exception(mock_slack, mock_logger):
    message = 'mock message'
    with pytest.raises(slack.errors.SlackApiError):
        Messages.send_slack_message(message)

    mock_logger.error.assert_called_once_with(
        "Could not send Slack message: The request to the Slack API failed.\nThe server responded with: {'ok': False, 'error': 'not_authed'}"  # noqa
    )


def test_prepare_github_message(_mock_pull_request, _mock_user, _mock_repo):
    result, discord_result = Messages.prepare_github_message(_mock_pull_request, False, False, False)

    assert 'Pull Request' in result
    assert f'{_mock_pull_request["assignees"][0]["html_url"]}|{_mock_pull_request["assignees"][0]["login"]}' in result
    assert f'{_mock_pull_request["html_url"]}|{_mock_pull_request["title"]}' in result


def test_prepare_github_message_no_assignee(_mock_pull_request):
    _mock_pull_request['assignees'] = []
    result, discord_result = Messages.prepare_github_message(_mock_pull_request, False, False, False)

    assert '*Waiting on:* NA' in result


def test_prepare_gitlab_message(_mock_merge_request, _mock_url, _mock_user, _mock_repo):
    result, discord_result = Messages.prepare_gitlab_message(_mock_merge_request, False, False, False)

    assert 'Merge Request' in result
    assert f'{_mock_merge_request["assignees"][0]["web_url"]}|{_mock_merge_request["assignees"][0]["username"]}' in result  # noqa
    assert f'{_mock_merge_request["web_url"]}|{_mock_merge_request["title"]}' in result


def test_prepare_gitlab_message_no_assignee(_mock_merge_request):
    _mock_merge_request['assignees'] = []
    result, discord_result = Messages.prepare_gitlab_message(_mock_merge_request, False, False, False)

    assert '*Waiting on:* NA' in result



import logging
import os

import requests

from pullbug.logger import PullBugLogger
from pullbug.messages import Messages

GITHUB_TOKEN = os.getenv('GITHUB_TOKEN')
GITHUB_HEADERS = {
    'Authorization': f'token {GITHUB_TOKEN}',
    'Content-Type': 'application/json; charset=utf-8'
}
LOGGER = logging.getLogger(__name__)


class GithubBug():
    @classmethod
    def run(cls, github_owner=None, github_state='open', github_context='orgs', wip=False,
            discord=False, slack=False, rocketchat=False):
        """Run the logic to get PR's from GitHub and
        send that data via message.
        """
        PullBugLogger._setup_logging(LOGGER)
        repos = cls.get_repos(github_owner, github_context)
        pull_requests = cls.get_pull_requests(repos, github_owner, github_state)

        if pull_requests == []:
            message = 'No pull requests are available from GitHub.'
            LOGGER.info(message)
            return message

        message_preamble = '\n:bug: *The following pull requests on GitHub are still open and need your help!*\n'
        messages, discord_messages = cls.iterate_pull_requests(pull_requests, wip, discord, slack, rocketchat)
        messages.insert(0, message_preamble)
        discord_messages.insert(0, message_preamble)
        if discord:
            Messages.send_discord_message(discord_messages)
        if slack:
            Messages.send_slack_message(messages)
        if rocketchat:
            Messages.send_rocketchat_message(messages)
        LOGGER.info(messages)

    @classmethod
    def get_repos(cls, github_owner, github_context=''):
        """Get all repos of the github_owner.
        """
        LOGGER.info('Bugging GitHub for repos...')
        try:
            response = requests.get(
                f'https://api.github.com/{github_context}/{github_owner}/repos?per_page=100',
                headers=GITHUB_HEADERS
            )
            LOGGER.debug(response.text)
            LOGGER.info('GitHub repos retrieved!')
            if 'Not Found' in response.text:
                error = f'Could not retrieve GitHub repos due to bad parameter: {github_owner} | {github_context}.'
                LOGGER.error(error)
                raise ValueError(error)
        except requests.exceptions.RequestException as response_error:
            LOGGER.error(
                f'Could not retrieve GitHub repos: {response_error}'
            )
            raise requests.exceptions.RequestException(response_error)
        return response.json()

    @classmethod
    def get_pull_requests(cls, repos, github_owner, github_state):
        """Grab all pull requests from each repo.
        """
        LOGGER.info('Bugging GitHub for pull requests...')
        pull_requests = []
        for repo in repos:
            try:
                pull_response = requests.get(
                    f'https://api.github.com/repos/{github_owner}/{repo["name"]}/pulls?state={github_state}&per_page=100',  # noqa
                    headers=GITHUB_HEADERS
                )
                if pull_response and pull_response.json():
                    LOGGER.debug(pull_response.text)
                    for single_pull_request in pull_response.json():
                        pull_requests.append(single_pull_request)
                else:
                    # Repo has no pull requests
                    continue
            except requests.exceptions.RequestException as response_error:
                LOGGER.error(
                    f'Could not retrieve GitHub pull requests for {repo["name"]}: {response_error}'
                )
                raise requests.exceptions.RequestException(response_error)
            except TypeError:
                error = f'Could not retrieve GitHub pull requests due to bad parameter: {github_owner} | {github_state}.'  # noqa
                LOGGER.error(error)
                raise TypeError(error)
        LOGGER.info('Pull requests retrieved!')
        return pull_requests

    @classmethod
    def iterate_pull_requests(cls, pull_requests, wip, discord, slack, rocketchat):
        """Iterate through each pull request of a repo
        and build the message array.
        """
        message_array = []
        discord_message_array = []
        for pull_request in pull_requests:
            if not wip and 'WIP' in pull_request['title'].upper():
                continue
            else:
                message, discord_message = Messages.prepare_github_message(pull_request, discord, slack, rocketchat)
                message_array.append(message)
                discord_message_array.append(discord_message)
        return message_array, discord_message_array



import pytest


@pytest.fixture
def _mock_github_token(scope='module'):
    return '123'


@pytest.fixture
def _mock_user():
    return 'mock-user'


@pytest.fixture
def _mock_github_context():
    return 'users'


@pytest.fixture
def _mock_github_state():
    return 'open'


@pytest.fixture
def _mock_repo():
    return {
        'name': 'mock-repo'
    }


@pytest.fixture
def _mock_pull_request(_mock_user, _mock_repo):
    return {
        'title': 'mock-pull-request',
        'description': 'Mock description',
        'assignees': [
            {
                'login': _mock_user,
                'html_url': f'https://github.com/{_mock_user}'
            },
        ],
        'body': 'Mock body of a pull request.',
        'html_url': f'https://github.com/{_mock_user}/{_mock_repo}/pull/1',
        'base': {
            'repo': {
                'name': 'mock-repo',
                'html_url': f'https://github.com/{_mock_user}/{_mock_repo}'
            }
        }
    }


@pytest.fixture
def _mock_url():
    return 'http://mock-url.com'


@pytest.fixture
def _mock_gitlab_scope():
    return 'all'


@pytest.fixture
def _mock_gitlab_state():
    return 'opened'


@pytest.fixture
def _mock_merge_request(_mock_user, _mock_repo, _mock_url):
    return {
        'title': 'mock-pull-request',
        'description': 'Mock description',
        'assignees': [
            {
                'username': _mock_user,
                'web_url': f'https://github.com/{_mock_user}'
            }
        ],
        'body': 'Mock body of a pull request.',
        'web_url': f'{_mock_url}/{_mock_user}/{_mock_repo}/pull/1',
        'references': {
            'short': '!1',
            'relative': '!1',
            'full': 'my-group/my-project!1'
        },
    }


class MockResponse():
    def __init__(self, json=None, text=None):
        self.json = json
        self.text = text



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

REQUIREMENTS = [
    'requests == 2.*',
    'slackclient == 2.*',
    'python-dotenv == 0.17.*'
]

DEV_REQUIREMENTS = [
    'coveralls == 3.*',
    'flake8',
    'mock == 4.*',
    'pytest == 6.*',
    'pytest-cov == 2.*',
]

setuptools.setup(
    name='pullbug',
    version='2.3.0',
    description='Get bugged via Discord, Slack, or RocketChat to merge your GitHub pull requests or GitLab merge requests.',  # noqa
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='http://github.com/justintime50/pull-bug',
    author='Justintime50',
    license='MIT',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=REQUIREMENTS,
    extras_require={
        'dev': DEV_REQUIREMENTS
    },
    entry_points={
        'console_scripts': [
            'pullbug=pullbug.cli:main'
        ]
    },
    python_requires='>=3.6',
)
